@@ -1,7 +1,7 @@
 /*
  *	pg_upgrade.h
  *
- *	Copyright (c) 2010-2013, PostgreSQL Global Development Group
+ *	Copyright (c) 2010-2014, PostgreSQL Global Development Group
  *	contrib/pg_upgrade/pg_upgrade.h
  */
 
@@ -11,7 +11,6 @@
 #include <sys/time.h>
 
 #include "libpq-fe.h"
-#include "pqexpbuffer.h"
 
 /* Use port in the private/dynamic port number range */
 #define DEF_PGUPORT			50432
@@ -82,7 +81,6 @@ extern char *output_files[];
 #else
 #define pg_mv_file			pgrename
 #define pg_link_file		win32_pghardlink
-#define sleep(x)			Sleep(x * 1000)
 #define PATH_SEPARATOR		'\\'
 #define RM_CMD				"DEL /q"
 #define RMDIR_CMD			"RMDIR /s/q"
@@ -107,6 +105,11 @@ extern char *output_files[];
  */
 #define VISIBILITY_MAP_CRASHSAFE_CAT_VER 201107031
 
+/*
+ * change in JSONB format during 9.4 beta
+ */
+#define JSONB_FORMAT_CHANGE_CAT_VER 201409291
+
 /*
  * pg_multixact format changed in 9.3 commit 0ac5ad5134f2769ccbaefec73844f85,
  * ("Improve concurrency of foreign key locking") which also updated catalog
@@ -126,7 +129,9 @@ typedef struct
 	Oid			reloid;			/* relation oid */
 	Oid			relfilenode;	/* relation relfile node */
 	/* relation tablespace path, or "" for the cluster default */
-	char		tablespace[MAXPGPATH];
+	char	   *tablespace;
+	bool		nsp_alloc;
+	bool		tblsp_alloc;
 } RelInfo;
 
 typedef struct
@@ -140,10 +145,10 @@ typedef struct
  */
 typedef struct
 {
-	char		old_tablespace[MAXPGPATH];
-	char		new_tablespace[MAXPGPATH];
-	char		old_tablespace_suffix[MAXPGPATH];
-	char		new_tablespace_suffix[MAXPGPATH];
+	const char *old_tablespace;
+	const char *new_tablespace;
+	const char *old_tablespace_suffix;
+	const char *new_tablespace_suffix;
 	Oid			old_db_oid;
 	Oid			new_db_oid;
 
@@ -165,7 +170,8 @@ typedef struct
 {
 	Oid			db_oid;			/* oid of the database */
 	char	   *db_name;		/* database name */
-	char		db_tblspace[MAXPGPATH]; /* database default tablespace path */
+	char		db_tablespace[MAXPGPATH];		/* database default tablespace
+												 * path */
 	RelInfoArr	rel_arr;		/* array of all user relinfos */
 } DbInfo;
 
@@ -255,7 +261,7 @@ typedef struct
 	Oid			pg_database_oid;	/* OID of pg_database relation */
 	Oid			install_role_oid;		/* OID of connected role */
 	Oid			role_count;		/* number of roles defined in the cluster */
-	char	   *tablespace_suffix;		/* directory specification */
+	const char *tablespace_suffix;		/* directory specification */
 } ClusterInfo;
 
 
@@ -290,6 +296,7 @@ typedef struct
 	const char *progname;		/* complete pathname for this program */
 	char	   *exec_path;		/* full path to my executable */
 	char	   *user;			/* username for clusters */
+	bool		user_specified; /* user specified on command-line */
 	char	  **old_tablespaces;	/* tablespaces */
 	int			num_old_tablespaces;
 	char	  **libraries;		/* loadable libraries */
@@ -315,7 +322,7 @@ void check_and_dump_old_cluster(bool live_check,
 						   char **sequence_script_file_name);
 void		check_new_cluster(void);
 void		report_clusters_compatible(void);
-void		issue_warnings_and_set_wal_level(char *sequence_script_file_name);
+void		issue_warnings(char *sequence_script_file_name);
 void output_completion_banner(char *analyze_script_file_name,
 						 char *deletion_script_file_name);
 void		check_cluster_versions(void);
@@ -334,6 +341,7 @@ void		disable_old_cluster(void);
 /* dump.c */
 
 void		generate_old_dump(void);
+void		optionally_create_toast_tables(void);
 
 
 /* exec.c */
@@ -441,23 +449,23 @@ void		check_pghost_envvar(void);
 /* util.c */
 
 char	   *quote_identifier(const char *s);
-extern void appendShellString(PQExpBuffer buf, const char *str);
-extern void appendConnStrVal(PQExpBuffer buf, const char *str);
-extern void appendPsqlMetaConnect(PQExpBuffer buf, const char *dbname);
-int			get_user_info(char **user_name);
+int			get_user_info(char **user_name_p);
 void		check_ok(void);
 void
 report_status(eLogType type, const char *fmt,...)
 __attribute__((format(PG_PRINTF_ATTRIBUTE, 2, 3)));
 void
-pg_log(eLogType type, char *fmt,...)
+pg_log(eLogType type, const char *fmt,...)
 __attribute__((format(PG_PRINTF_ATTRIBUTE, 2, 3)));
+void
+pg_fatal(const char *fmt,...)
+__attribute__((format(PG_PRINTF_ATTRIBUTE, 1, 2), noreturn));
 void		end_progress_output(void);
 void
 prep_status(const char *fmt,...)
 __attribute__((format(PG_PRINTF_ATTRIBUTE, 1, 2)));
 void		check_ok(void);
-const char *getErrorText(int errNum);
+const char *getErrorText(void);
 unsigned int str2uint(const char *str);
 void		pg_putenv(const char *var, const char *val);
 
@@ -466,6 +474,7 @@ void		pg_putenv(const char *var, const char *val);
 
 void new_9_0_populate_pg_largeobject_metadata(ClusterInfo *cluster,
 										 bool check_mode);
+void old_9_3_check_for_line_data_type_usage(ClusterInfo *cluster);
 
 /* version_old_8_3.c */
 
