@@ -6,7 +6,7 @@
  * See src/backend/optimizer/README for discussion of EquivalenceClasses.
  *
  *
- * Portions Copyright (c) 1996-2013, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2014, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
@@ -424,9 +424,8 @@ canonicalize_ec_expression(Expr *expr, Oid req_type, Oid req_collation)
 
 	/*
 	 * For a polymorphic-input-type opclass, just keep the same exposed type.
-	 * RECORD opclasses work like polymorphic-type ones for this purpose.
 	 */
-	if (IsPolymorphicType(req_type) || req_type == RECORDOID)
+	if (IsPolymorphicType(req_type))
 		req_type = expr_type;
 
 	/*
@@ -989,31 +988,13 @@ generate_base_implied_equalities_broken(PlannerInfo *root,
  *
  * join_relids should always equal bms_union(outer_relids, inner_rel->relids).
  * We could simplify this function's API by computing it internally, but in
- * most current uses, the caller has the value at hand anyway.
+ * all current uses, the caller has the value at hand anyway.
  */
 List *
 generate_join_implied_equalities(PlannerInfo *root,
 								 Relids join_relids,
 								 Relids outer_relids,
 								 RelOptInfo *inner_rel)
-{
-	return generate_join_implied_equalities_for_ecs(root,
-													root->eq_classes,
-													join_relids,
-													outer_relids,
-													inner_rel);
-}
-
-/*
- * generate_join_implied_equalities_for_ecs
- *	  As above, but consider only the listed ECs.
- */
-List *
-generate_join_implied_equalities_for_ecs(PlannerInfo *root,
-										 List *eclasses,
-										 Relids join_relids,
-										 Relids outer_relids,
-										 RelOptInfo *inner_rel)
 {
 	List	   *result = NIL;
 	Relids		inner_relids = inner_rel->relids;
@@ -1035,7 +1016,7 @@ generate_join_implied_equalities_for_ecs(PlannerInfo *root,
 		nominal_join_relids = join_relids;
 	}
 
-	foreach(lc, eclasses)
+	foreach(lc, root->eq_classes)
 	{
 		EquivalenceClass *ec = (EquivalenceClass *) lfirst(lc);
 		List	   *sublist = NIL;
@@ -1959,8 +1940,8 @@ add_child_rel_equivalences(PlannerInfo *root,
 			continue;
 
 		/*
-		 * No point in searching if parent rel not mentioned in eclass; but
-		 * we can't tell that for sure if parent rel is itself a child.
+		 * No point in searching if parent rel not mentioned in eclass; but we
+		 * can't tell that for sure if parent rel is itself a child.
 		 */
 		if (parent_rel->reloptkind == RELOPT_BASEREL &&
 			!bms_is_subset(parent_rel->relids, cur_ec->ec_relids))
