@@ -3,11 +3,11 @@
  * int8.c
  *	  Internal 64-bit integer operations
  *
- * Portions Copyright (c) 1996-2009, PostgreSQL Global Development Group
+ * Portions Copyright (c) 1996-2014, PostgreSQL Global Development Group
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $PostgreSQL: pgsql/src/backend/utils/adt/int8.c,v 1.74.2.1 2009/09/03 18:48:21 tgl Exp $
+ *	  src/backend/utils/adt/int8.c
  *
  *-------------------------------------------------------------------------
  */
@@ -19,8 +19,8 @@
 
 #include "funcapi.h"
 #include "libpq/pqformat.h"
-#include "nodes/nodes.h"
 #include "utils/int8.h"
+#include "utils/builtins.h"
 
 
 #define MAXINT8LEN		25
@@ -76,15 +76,12 @@ scanint8(const char *str, bool errorOK, int64 *result)
 		 * Do an explicit check for INT64_MIN.  Ugly though this is, it's
 		 * cleaner than trying to get the loop below to handle it portably.
 		 */
-#ifndef INT64_IS_BUSTED
 		if (strncmp(ptr, "9223372036854775808", 19) == 0)
 		{
 			tmp = -INT64CONST(0x7fffffffffffffff) - 1;
 			ptr += 19;
 			goto gotdigits;
 		}
-#endif
-
 		sign = -1;
 	}
 	else if (*ptr == '+')
@@ -161,13 +158,10 @@ Datum
 int8out(PG_FUNCTION_ARGS)
 {
 	int64		val = PG_GETARG_INT64(0);
-	char	   *result;
-	int			len;
 	char		buf[MAXINT8LEN + 1];
+	char	   *result;
 
-	if ((len = snprintf(buf, MAXINT8LEN, INT64_FORMAT, val)) < 0)
-		elog(ERROR, "could not format int8");
-
+	pg_lltoa(val, buf);
 	result = pstrdup(buf);
 	PG_RETURN_CSTRING(result);
 }
@@ -575,12 +569,9 @@ int8mul(PG_FUNCTION_ARGS)
 	 * Since the division is likely much more expensive than the actual
 	 * multiplication, we'd like to skip it where possible.  The best bang for
 	 * the buck seems to be to check whether both inputs are in the int32
-	 * range; if so, no overflow is possible.  (But that only works if we
-	 * really have a 64-bit int64 datatype...)
+	 * range; if so, no overflow is possible.
 	 */
-#ifndef INT64_IS_BUSTED
 	if (arg1 != (int64) ((int32) arg1) || arg2 != (int64) ((int32) arg2))
-#endif
 	{
 		if (arg2 != 0 &&
 			((arg2 == -1 && arg1 < 0 && result < 0) ||
@@ -687,15 +678,13 @@ int8inc(PG_FUNCTION_ARGS)
 {
 	/*
 	 * When int8 is pass-by-reference, we provide this special case to avoid
-	 * palloc overhead for COUNT(): when called from nodeAgg, we know that the
-	 * argument is modifiable local storage, so just update it in-place. (If
-	 * int8 is pass-by-value, then of course this is useless as well as
+	 * palloc overhead for COUNT(): when called as an aggregate, we know that
+	 * the argument is modifiable local storage, so just update it in-place.
+	 * (If int8 is pass-by-value, then of course this is useless as well as
 	 * incorrect, so just ifdef it out.)
 	 */
 #ifndef USE_FLOAT8_BYVAL		/* controls int8 too */
-	if (fcinfo->context &&
-		(IsA(fcinfo->context, AggState) ||
-		 IsA(fcinfo->context, WindowAggState)))
+	if (AggCheckCallContext(fcinfo, NULL))
 	{
 		int64	   *arg = (int64 *) PG_GETARG_POINTER(0);
 		int64		result;
@@ -713,7 +702,7 @@ int8inc(PG_FUNCTION_ARGS)
 	else
 #endif
 	{
-		/* Not called by nodeAgg, so just do it the dumb way */
+		/* Not called as an aggregate, so just do it the dumb way */
 		int64		arg = PG_GETARG_INT64(0);
 		int64		result;
 
@@ -728,13 +717,58 @@ int8inc(PG_FUNCTION_ARGS)
 	}
 }
 
+Datum
+int8dec(PG_FUNCTION_ARGS)
+{
+	/*
+	 * When int8 is pass-by-reference, we provide this special case to avoid
+	 * palloc overhead for COUNT(): when called as an aggregate, we know that
+	 * the argument is modifiable local storage, so just update it in-place.
+	 * (If int8 is pass-by-value, then of course this is useless as well as
+	 * incorrect, so just ifdef it out.)
+	 */
+#ifndef USE_FLOAT8_BYVAL		/* controls int8 too */
+	if (AggCheckCallContext(fcinfo, NULL))
+	{
+		int64	   *arg = (int64 *) PG_GETARG_POINTER(0);
+		int64		result;
+
+		result = *arg - 1;
+		/* Overflow check */
+		if (result > 0 && *arg < 0)
+			ereport(ERROR,
+					(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+					 errmsg("bigint out of range")));
+
+		*arg = result;
+		PG_RETURN_POINTER(arg);
+	}
+	else
+#endif
+	{
+		/* Not called as an aggregate, so just do it the dumb way */
+		int64		arg = PG_GETARG_INT64(0);
+		int64		result;
+
+		result = arg - 1;
+		/* Overflow check */
+		if (result > 0 && arg < 0)
+			ereport(ERROR,
+					(errcode(ERRCODE_NUMERIC_VALUE_OUT_OF_RANGE),
+					 errmsg("bigint out of range")));
+
+		PG_RETURN_INT64(result);
+	}
+}
+
+
 /*
- * These functions are exactly like int8inc but are used for aggregates that
- * count only non-null values.  Since the functions are declared strict,
- * the null checks happen before we ever get here, and all we need do is
- * increment the state value.  We could actually make these pg_proc entries
- * point right at int8inc, but then the opr_sanity regression test would
- * complain about mismatched entries for a built-in function.
+ * These functions are exactly like int8inc/int8dec but are used for
+ * aggregates that count only non-null values.  Since the functions are
+ * declared strict, the null checks happen before we ever get here, and all we
+ * need do is increment the state value.  We could actually make these pg_proc
+ * entries point right at int8inc/int8dec, but then the opr_sanity regression
+ * test would complain about mismatched entries for a built-in function.
  */
 
 Datum
@@ -749,6 +783,12 @@ int8inc_float8_float8(PG_FUNCTION_ARGS)
 	return int8inc(fcinfo);
 }
 
+Datum
+int8dec_any(PG_FUNCTION_ARGS)
+{
+	return int8dec(fcinfo);
+}
+
 
 Datum
 int8larger(PG_FUNCTION_ARGS)
